﻿
Namespace GamepadData

    Public Class Gamepad
        Inherits NotifyPropertyChanged

        Dim _PlayerNumber As PlayerNumber
        Public ReadOnly Property PlayerNumber As PlayerNumber
            Get
                Return _PlayerNumber
            End Get
        End Property

        Dim _XInputPlayerIndex As XInput.PlayerIndex
        Public ReadOnly Property XInputPlayerIndex As XInput.PlayerIndex
            <DebuggerStepThrough()>
            Get
                Return _XInputPlayerIndex
            End Get
        End Property

        Dim _IsConnected As Boolean = False
        Public Property IsConnected As Boolean
            Get
                Return _IsConnected
            End Get
            Private Set(value As Boolean)
                ExtendedChangeIfDifferent(_IsConnected, value, "IsConnected")
            End Set
        End Property

        Dim _IsActive As Boolean = False
        Public Property IsActive As Boolean
            Get
                Return _IsActive
            End Get
            Set(value As Boolean)
                ExtendedChangeIfDifferent(_IsActive, value, "IsActive")
            End Set
        End Property

        Dim _Configuration As KeyBinding.Configurations.KeyBindingConfiguration = Nothing
        Public Property Configuration As KeyBinding.Configurations.KeyBindingConfiguration
            Get
                Return _Configuration
            End Get
            Set(value As KeyBinding.Configurations.KeyBindingConfiguration)
                ExtendedChangeIfDifferent(_Configuration, value, "Configuration")
            End Set
        End Property

        Dim _TemporaryInterceptor As ITemporaryGamepadStateInterceptor = Nothing
        Public Property TemporaryInterceptor As ITemporaryGamepadStateInterceptor
            Get
                Return _TemporaryInterceptor
            End Get
            Set(value As ITemporaryGamepadStateInterceptor)
                ExtendedChangeIfDifferent(_TemporaryInterceptor, value, "TemporaryInterceptor")
            End Set
        End Property

        Public Sub New(NewPlayerNumber As PlayerNumber)
            _PlayerNumber = NewPlayerNumber
            _XInputPlayerIndex = _PlayerNumber.GetFieldAttribute(Of XInputPlayerIndexAttribute)().PlayerIndex
        End Sub

        Private Function GetCurrentXnaState() As XInput.IGamepadState
            If ViewModelBase.IsInDesignMode Then
                Return Nothing
            End If
            Return XInput.Gamepad.GetState(XInputPlayerIndex)
        End Function

        Public Sub UpdateStateAndApplyConfigurationForButtonActions()
            If ViewModelBase.IsInDesignMode Then
                Return
            End If
            Dim CurrentState = GetCurrentXnaState()
            IsConnected = CurrentState.IsConnected
            If Not IsActive OrElse Configuration Is Nothing Then
                Return
            End If
            If TemporaryInterceptor IsNot Nothing Then
                TemporaryInterceptor.Intercept(Me, CurrentState)
                Return
            End If
            If IsConnected Then
                For Each i In Configuration.Actions
                    Dim Kind = i.Input.GetFieldAttribute(Of GamepadInputKindAttribute)().Kind
                    Select Case Kind
                        Case GamepadInputKind.Button
                            Dim CastedAction = DirectCast(i.Action, KeyBinding.Actions.Button.ButtonKeyBindingActionBase)
                            Dim XnaButton = i.Input.GetFieldAttribute(Of XInputButtonAttribute)().Button
                            CastedAction.UpdateIsButtonDown(CurrentState.GetButtonIsDown(XnaButton), Me)
                        Case GamepadInputKind.Stick
                            Dim CastedAction = DirectCast(i.Action, KeyBinding.Actions.Stick.StickKeyBindingActionBase)
                            Dim XnaStick = i.Input.GetFieldAttribute(Of XInputStickAttribute)().Stick
                            CastedAction.UpdateStickValue(CurrentState.GetStickValue(XnaStick), Me)
                        Case GamepadInputKind.Trigger
                            Dim CastedAction = DirectCast(i.Action, KeyBinding.Actions.Trigger.TriggerKeyBindingAction)
                            Dim XnaTrigger = i.Input.GetFieldAttribute(Of XInputTriggerAttribute)().Trigger
                            CastedAction.UpdateTriggerValue(CurrentState.GetTriggerValue(XnaTrigger), Me)
                    End Select
                Next
            End If
        End Sub

    End Class

End Namespace
