﻿
Namespace KeyBinding.Actions.Button

    <DisplayName("Konfiguration wechseln")>
    <KeyBindingActionInputKind(GamepadData.GamepadInputKind.Button)>
        <KeyBinding.Serialization.SerializableThing("ChangeConfigurationAction")>
    Public Class ChangeConfigurationAction
        Inherits ButtonKeyBindingActionBase

        Dim _TargetConfiguration As KeyBinding.Configurations.KeyBindingConfiguration = Nothing
        Public Property TargetConfiguration As KeyBinding.Configurations.KeyBindingConfiguration
            Get
                Return _TargetConfiguration
            End Get
            Set(value As KeyBinding.Configurations.KeyBindingConfiguration)
                ExtendedChangeIfDifferent(_TargetConfiguration, value, "TargetConfiguration")
            End Set
        End Property

        Protected Overrides Sub OnIsButtonDownChanged(IsButtonDown As Boolean, SenderGamepad As GamepadData.Gamepad)
            If Not IsButtonDown Then
                OnPublicAction(New ChangeConfigurationRequestedEventArgs(SenderGamepad, Me, TargetConfiguration))
            End If
        End Sub

        Public Overrides Sub ReplaceReferencedConfiguration(ToReplace As KeyBinding.Configurations.KeyBindingConfiguration, ReplaceWith As KeyBinding.Configurations.KeyBindingConfiguration)
            If TargetConfiguration Is ToReplace Then
                TargetConfiguration = ReplaceWith
            End If
        End Sub

        Public Overrides Function Clone() As KeyBindingAction
            Return New ChangeConfigurationAction With {.TargetConfiguration = Me.TargetConfiguration}
        End Function

#Region "Serialization"

        Public Sub New()
        End Sub

        Public Sub New(Context As KeyBinding.Serialization.DeserializationContext, Source As XElement)
            Dim TargetConfigurationGuid = Source.TryGetAttributeGuid("TargetConfiguration")
            If TargetConfigurationGuid IsNot Nothing Then
                TargetConfiguration = Context.GetConfigurationByIdentifier(TargetConfigurationGuid.Value)
            End If
        End Sub

        Public Overrides Function Serialize(Context As KeyBinding.Serialization.SerializationContext) As XElement
            Return <ChangeConfigurationAction
                       TargetConfiguration=<%= If(TargetConfiguration Is Nothing, Nothing, DirectCast(Context.GetConfigurationIdentifier(TargetConfiguration), Object)) %>
                   />
        End Function

#End Region

        Public Class ChangeConfigurationRequestedEventArgs
            Inherits KeyBindingPublicActionEventArgs

            Dim _NewConfiguration As KeyBinding.Configurations.KeyBindingConfiguration
            Public ReadOnly Property NewConfiguration As KeyBinding.Configurations.KeyBindingConfiguration
                Get
                    Return _NewConfiguration
                End Get
            End Property

            Public Sub New(NewSenderGamepad As GamepadData.Gamepad, NewSenderAction As KeyBindingAction, NewNewConfiguration As KeyBinding.Configurations.KeyBindingConfiguration)
                MyBase.New(NewSenderGamepad, NewSenderAction)
                _NewConfiguration = NewNewConfiguration
            End Sub

        End Class

    End Class

End Namespace
