﻿
Imports GamepadControl.Native.SendInput

Namespace KeyBinding.Actions.Button

    <DisplayName("Tastaturtaste")>
    <KeyBindingActionInputKind(GamepadData.GamepadInputKind.Button)>
    <KeyBinding.Serialization.SerializableThing("KeyboardKeyAction")>
    Public Class KeyboardKeyAction
        Inherits ButtonKeyBindingActionBase

        Dim _Mode As KeyboardMode = KeyboardMode.VirtualKey
        Public Property Mode As KeyboardMode
            Get
                Return _Mode
            End Get
            Set(value As KeyboardMode)
                ExtendedChangeIfDifferent(_Mode, value, "Mode")
            End Set
        End Property

        Dim _VirtualKey As Native.SendInput.VirtualKey = Native.SendInput.VirtualKey.A
        Public Property VirtualKey As Native.SendInput.VirtualKey
            Get
                Return _VirtualKey
            End Get
            Set(value As Native.SendInput.VirtualKey)
                ExtendedChangeIfDifferent(_VirtualKey, value, "VirtualKey")
            End Set
        End Property

        Dim _Character As Char = "a"c
        Public Property Character As Char
            Get
                Return _Character
            End Get
            Set(value As Char)
                ExtendedChangeIfDifferent(_Character, value, "Character")
            End Set
        End Property

        Protected Overrides Sub OnIsButtonDownChanged(IsButtonDown As Boolean, SenderGamepad As GamepadData.Gamepad)
            Dim KeyboardInput As New KeyboardInput
            If Mode = KeyboardMode.VirtualKey Then
                KeyboardInput.VirtualKey = VirtualKey
            End If
            If Mode = KeyboardMode.Character Then
                KeyboardInput.Flags = KeyboardInput.Flags Or KeyEventFlags.Unicode
                KeyboardInput.Scan = Convert.ToInt16(Character)
            End If
            If Not IsButtonDown Then
                KeyboardInput.Flags = KeyboardInput.Flags Or KeyEventFlags.KeyUp
            End If
            NativeMethods.DispatchSendInput( _
                New Input With _
                {
                    .Type = Type.Keyboard, _
                    .Data = New Data With _
                    {
                        .KeyboardInput = KeyboardInput
                    }
                })
        End Sub

        Public Overrides Function Clone() As KeyBindingAction
            Return New KeyboardKeyAction With {.Mode = Me.Mode, .VirtualKey = Me.VirtualKey, .Character = Me.Character}
        End Function

#Region "Serialization"

        Public Sub New()
        End Sub

        Public Sub New(Context As KeyBinding.Serialization.DeserializationContext, Source As XElement)
            Mode = Source.GetAttributeEnum(Of KeyboardMode)("Mode")
            VirtualKey = Source.GetAttributeEnum(Of Native.SendInput.VirtualKey)("VirtualKey")
            Character = Source.GetAttributeChar("Character")
        End Sub

        Public Overrides Function Serialize(Context As KeyBinding.Serialization.SerializationContext) As XElement
            Return <KeyboardKeyAction
                       Mode=<%= Mode %>
                       VirtualKey=<%= VirtualKey %>
                       Character=<%= Character %>
                   />
        End Function

#End Region

    End Class

End Namespace
