﻿
Namespace KeyBinding.Actions.Button

    <DisplayName("Auswahl aus mehreren Aktionen")>
    <KeyBindingActionInputKind(GamepadData.GamepadInputKind.Button)>
    <KeyBinding.Serialization.SerializableThing("MultiAction")>
    Public Class MultiAction
        Inherits ButtonKeyBindingActionBase

        Dim WithEvents _UpActions As New TypedObservableCollection(Of MultiActionSubAction)
        Public ReadOnly Property UpActions As TypedObservableCollection(Of MultiActionSubAction)
            Get
                Return _UpActions
            End Get
        End Property

        Dim WithEvents _DownActions As New TypedObservableCollection(Of MultiActionSubAction)
        Public ReadOnly Property DownActions As TypedObservableCollection(Of MultiActionSubAction)
            Get
                Return _DownActions
            End Get
        End Property

        Dim WithEvents _LeftActions As New TypedObservableCollection(Of MultiActionSubAction)
        Public ReadOnly Property LeftActions As TypedObservableCollection(Of MultiActionSubAction)
            Get
                Return _LeftActions
            End Get
        End Property

        Dim WithEvents _RightActions As New TypedObservableCollection(Of MultiActionSubAction)
        Public ReadOnly Property RightActions As TypedObservableCollection(Of MultiActionSubAction)
            Get
                Return _RightActions
            End Get
        End Property

        Private Sub Actions_ItemAdded(sender As Object, e As ItemEventArgs(Of MultiActionSubAction)) Handles _UpActions.ItemAdded, _DownActions.ItemAdded, _LeftActions.ItemAdded, _RightActions.ItemAdded
            AddHandler e.Item.PublicAction, AddressOf Action_PublicAction
        End Sub
        Private Sub Actions_ItemRemoved(sender As Object, e As ItemEventArgs(Of MultiActionSubAction)) Handles _UpActions.ItemRemoved, _DownActions.ItemRemoved, _LeftActions.ItemRemoved, _RightActions.ItemRemoved
            RemoveHandler e.Item.PublicAction, AddressOf Action_PublicAction
        End Sub

        Private Sub Action_PublicAction(sender As Object, e As KeyBindingPublicActionEventArgs)
            OnPublicAction(e)
        End Sub

        Protected Overrides Sub OnIsButtonDownChanged(IsButtonDown As Boolean, SenderGamepad As GamepadData.Gamepad)
            If Not IsButtonDown Then
                OnPublicAction(New MultiActionInvokedEventArgs(SenderGamepad, Me))
            End If
        End Sub

        Public Overrides Sub ReplaceReferencedConfiguration(ToReplace As KeyBinding.Configurations.KeyBindingConfiguration, ReplaceWith As KeyBinding.Configurations.KeyBindingConfiguration)
            For Each i In UpActions
                i.Action.ReplaceReferencedConfiguration(ToReplace, ReplaceWith)
            Next
            For Each i In DownActions
                i.Action.ReplaceReferencedConfiguration(ToReplace, ReplaceWith)
            Next
            For Each i In LeftActions
                i.Action.ReplaceReferencedConfiguration(ToReplace, ReplaceWith)
            Next
            For Each i In RightActions
                i.Action.ReplaceReferencedConfiguration(ToReplace, ReplaceWith)
            Next
        End Sub

        Public Overrides Function Clone() As KeyBindingAction
            Dim Result As New MultiAction
            Result.UpActions.AddRange(Me.UpActions.Select(Function(i) i.Clone))
            Result.DownActions.AddRange(Me.DownActions.Select(Function(i) i.Clone))
            Result.LeftActions.AddRange(Me.LeftActions.Select(Function(i) i.Clone))
            Result.RightActions.AddRange(Me.RightActions.Select(Function(i) i.Clone))
            Return Result
        End Function

#Region "Serialization"

        Public Sub New()
        End Sub

        Public Sub New(Context As KeyBinding.Serialization.DeserializationContext, Source As XElement)
            Dim DeserializeSubAction = Function(i As XElement)
                                           Dim Action = Context.DeserializeAction(i.Elements.Single)
                                           If Not TypeOf Action Is ButtonKeyBindingActionBase Then
                                               Throw New KeyBinding.Serialization.DeserializationException("Actions in MultiAction müssen von ButtonKeyBindingActionBase erben.")
                                           End If
                                           Return New MultiActionSubAction With {.Name = i.GetAttributeString("Name"), .Action = DirectCast(Action, ButtonKeyBindingActionBase)}
                                       End Function
            UpActions.AddRange(Source.<UpActions>.Elements.Select(DeserializeSubAction))
            DownActions.AddRange(Source.<DownActions>.Elements.Select(DeserializeSubAction))
            LeftActions.AddRange(Source.<LeftActions>.Elements.Select(DeserializeSubAction))
            RightActions.AddRange(Source.<RightActions>.Elements.Select(DeserializeSubAction))
        End Sub

        Public Overrides Function Serialize(Context As KeyBinding.Serialization.SerializationContext) As XElement
            Return <MultiAction>
                       <UpActions>
                           <%= UpActions.Select(Function(i) <MultiActionSubAction
                                                                Name=<%= i.Name %>
                                                                >
                                                                <%= i.Action.Serialize(Context) %>
                                                            </MultiActionSubAction>) %>
                       </UpActions>
                       <DownActions>
                           <%= DownActions.Select(Function(i) <MultiActionSubAction
                                                                  Name=<%= i.Name %>
                                                                  >
                                                                  <%= i.Action.Serialize(Context) %>
                                                              </MultiActionSubAction>) %>
                       </DownActions>
                       <LeftActions>
                           <%= LeftActions.Select(Function(i) <MultiActionSubAction
                                                                  Name=<%= i.Name %>
                                                                  >
                                                                  <%= i.Action.Serialize(Context) %>
                                                              </MultiActionSubAction>) %>
                       </LeftActions>
                       <RightActions>
                           <%= RightActions.Select(Function(i) <MultiActionSubAction
                                                                   Name=<%= i.Name %>
                                                                   >
                                                                   <%= i.Action.Serialize(Context) %>
                                                               </MultiActionSubAction>) %>
                       </RightActions>
                   </MultiAction>
        End Function

#End Region

        Public Class MultiActionInvokedEventArgs
            Inherits KeyBindingPublicActionEventArgs

            Public Sub New(NewSenderGamepad As GamepadData.Gamepad, NewSenderAction As KeyBindingAction)
                MyBase.New(NewSenderGamepad, NewSenderAction)
            End Sub

        End Class

    End Class

End Namespace
