﻿
Namespace KeyBinding.Actions.Button

    <DisplayName("Mehrere Aktionen in Folge")>
    <KeyBindingActionInputKind(GamepadData.GamepadInputKind.Button)>
    <KeyBinding.Serialization.SerializableThing("SequenceAction")>
    Public Class SequenceAction
        Inherits ButtonKeyBindingActionBase

        Dim _ReverseOrderOnKeyUp As Boolean = True
        Public Property ReverseOrderOnKeyUp As Boolean
            Get
                Return _ReverseOrderOnKeyUp
            End Get
            Set(value As Boolean)
                ExtendedChangeIfDifferent(_ReverseOrderOnKeyUp, value, "ReverseOrderOnKeyUp")
            End Set
        End Property

        Dim WithEvents _Actions As New TypedObservableCollection(Of SequenceActionSubAction)
        Public ReadOnly Property Actions As TypedObservableCollection(Of SequenceActionSubAction)
            Get
                Return _Actions
            End Get
        End Property
        Private Sub Actions_ItemAdded(sender As Object, e As ItemEventArgs(Of SequenceActionSubAction)) Handles _Actions.ItemAdded
            AddHandler e.Item.PublicAction, AddressOf Action_PublicAction
        End Sub
        Private Sub Actions_ItemRemoved(sender As Object, e As ItemEventArgs(Of SequenceActionSubAction)) Handles _Actions.ItemRemoved
            RemoveHandler e.Item.PublicAction, AddressOf Action_PublicAction
        End Sub

        Private Sub Action_PublicAction(sender As Object, e As KeyBindingPublicActionEventArgs)
            OnPublicAction(e)
        End Sub

        Protected Overrides Sub OnIsButtonDownChanged(IsButtonDown As Boolean, SenderGamepad As GamepadData.Gamepad)
            If IsButtonDown Then
                For Each i In Actions
                    i.Action.UpdateIsButtonDown(True, SenderGamepad)
                Next
            Else
                For Each i In If(ReverseOrderOnKeyUp, Actions.Reverse, Actions)
                    i.Action.UpdateIsButtonDown(False, SenderGamepad)
                Next
            End If
        End Sub

        Public Overrides Sub ReplaceReferencedConfiguration(ToReplace As KeyBinding.Configurations.KeyBindingConfiguration, ReplaceWith As KeyBinding.Configurations.KeyBindingConfiguration)
            For Each i In Actions
                i.Action.ReplaceReferencedConfiguration(ToReplace, ReplaceWith)
            Next
        End Sub

        Public Overrides Function Clone() As KeyBindingAction
            Dim Result As New SequenceAction With {.ReverseOrderOnKeyUp = Me.ReverseOrderOnKeyUp}
            Result.Actions.AddRange(Me.Actions.Select(Function(i) i.Clone))
            Return Result
        End Function

#Region "Serialization"

        Public Sub New()
        End Sub

        Public Sub New(Context As KeyBinding.Serialization.DeserializationContext, Source As XElement)
            ReverseOrderOnKeyUp = Source.GetAttributeBoolean("ReverseOrderOnKeyUp")
            For Each i In Source.<Actions>.Elements
                Dim Action = Context.DeserializeAction(i)
                If Not TypeOf Action Is ButtonKeyBindingActionBase Then
                    Throw New KeyBinding.Serialization.DeserializationException("Aktionen in SequenceAction müssen von ButtonKeyBindingActionBase erben.")
                End If
                Actions.Add(New SequenceActionSubAction With {.Action = DirectCast(Action, ButtonKeyBindingActionBase)})
            Next
        End Sub

        Public Overrides Function Serialize(Context As KeyBinding.Serialization.SerializationContext) As XElement
            Return <SequenceAction
                       ReverseOrderOnKeyUp=<%= ReverseOrderOnKeyUp %>
                       >
                       <Actions>
                           <%= Actions.Select(Function(i) i.Action.Serialize(Context)) %>
                       </Actions>
                   </SequenceAction>
        End Function

#End Region

    End Class

End Namespace
