﻿
Namespace KeyBinding.Actions.Stick

    Public MustInherit Class StickKeyBindingActionBase
        Inherits KeyBindingAction
        Implements IContinuousAction

        Private Class MovementData
            Public VelocityX As Double
            Public VelocityY As Double
            Public CumulativeMovementX As Double = 0
            Public CumulativeMovementY As Double = 0
        End Class

        Dim _SlopeMode As SlopeMode = Actions.SlopeMode.Linear
        Public Property SlopeMode As SlopeMode
            Get
                Return _SlopeMode
            End Get
            Set(value As SlopeMode)
                ExtendedChangeIfDifferent(_SlopeMode, value, "SlopeMode")
            End Set
        End Property

        Dim _Speed As Double = 100
        Public Property Speed As Double
            Get
                Return _Speed
            End Get
            Set(value As Double)
                ExtendedChangeIfDifferent(_Speed, value, "Speed")
            End Set
        End Property

        Dim _InvertX As Boolean = False
        Public Property InvertX As Boolean
            Get
                Return _InvertX
            End Get
            Set(value As Boolean)
                ExtendedChangeIfDifferent(_InvertX, value, "InvertX")
            End Set
        End Property

        Dim _InvertY As Boolean = False
        Public Property InvertY As Boolean
            Get
                Return _InvertY
            End Get
            Set(value As Boolean)
                ExtendedChangeIfDifferent(_InvertY, value, "InvertY")
            End Set
        End Property

        Public Sub UpdateStickValue(StickValue As Vector2D, SenderGamepad As GamepadData.Gamepad)
            Dim MovementValue = GetMovementValue(StickValue)
            With MovementValues.GetOrNew(SenderGamepad)
                .VelocityX = MovementValue.X
                .VelocityY = MovementValue.Y
            End With
        End Sub

        Protected MovementFactor As New Vector2D(1, 1)

        <MultithreadingAwareness()>
        Dim MovementValues As New Dictionary(Of GamepadData.Gamepad, MovementData)

        <MultithreadingAwareness()>
        Public Sub ApplyAction(SenderGamepad As GamepadData.Gamepad, TimePassed As Double) Implements IContinuousAction.ApplyAction
            Dim MovementData = MovementValues.GetOrNew(SenderGamepad)
            If MovementData.VelocityX = 0 AndAlso MovementData.VelocityY = 0 Then
                Return
            End If
            MovementData.CumulativeMovementX += MovementData.VelocityX * TimePassed
            MovementData.CumulativeMovementY += MovementData.VelocityY * TimePassed
            Dim MovementX As Integer = 0
            Dim MovementY As Integer = 0
            If MovementData.CumulativeMovementX >= 1 Then
                Dim Temp = Math.Floor(MovementData.CumulativeMovementX)
                MovementX = CInt(Temp)
                MovementData.CumulativeMovementX -= Temp
            ElseIf MovementData.CumulativeMovementX <= -1 Then
                Dim Temp = Math.Ceiling(MovementData.CumulativeMovementX)
                MovementX = CInt(Temp)
                MovementData.CumulativeMovementX -= Temp
            End If
            If MovementData.CumulativeMovementY >= 1 Then
                Dim Temp = Math.Floor(MovementData.CumulativeMovementY)
                MovementY = CInt(Temp)
                MovementData.CumulativeMovementY -= Temp
            ElseIf MovementData.CumulativeMovementY <= 1 Then
                Dim Temp = Math.Ceiling(MovementData.CumulativeMovementY)
                MovementY = CInt(Temp)
                MovementData.CumulativeMovementY -= Temp
            End If
            ApplyActionCore(MovementX, MovementY)
        End Sub

        <MultithreadingAwareness()>
        Protected MustOverride Sub ApplyActionCore(MovementX As Integer, MovementY As Integer)

        Private Function GetMovementValue(StickValue As Vector2D) As Vector2D
            If StickValue.X = 0 AndAlso StickValue.Y = 0 Then
                Return New Vector2D(0, 0)
            End If
            Dim InvertValue As New Vector2D(If(InvertX, -1, 1), If(InvertY, -1, 1))
            Dim InputValue = Vector2D.ComponentwiseProduct(StickValue, InvertValue)
            Dim InputLength = InputValue.Length
            Dim InputAngle = -InputValue.Angle
            Dim MovementSpeed = InputLength ^ SlopeMode.GetFieldAttribute(Of SlopeModeExponentAttribute).Exponent * Speed
            Return Vector2D.ComponentwiseProduct(Vector2D.FromPolar(MovementSpeed, InputAngle), MovementFactor)
        End Function

        Public Overrides Function Clone() As KeyBindingAction
            Dim Result = CloneStickKeyBindingActionBase()
            Result.SlopeMode = Me.SlopeMode
            Result.Speed = Me.Speed
            Result.InvertX = Me.InvertX
            Result.InvertY = Me.InvertY
            Return Result
        End Function

        Protected MustOverride Function CloneStickKeyBindingActionBase() As StickKeyBindingActionBase

#Region "Serialization"

        Public Sub New()
        End Sub

        Public Sub New(Context As KeyBinding.Serialization.DeserializationContext, Source As XElement)
            SlopeMode = Source.GetAttributeEnum(Of SlopeMode)("SlopeMode")
            Speed = Source.GetAttributeDouble("Speed")
            InvertX = Source.GetAttributeBoolean("InvertX")
            InvertY = Source.GetAttributeBoolean("InvertY")
        End Sub

        Public Overrides Function Serialize(Context As KeyBinding.Serialization.SerializationContext) As XElement
            Return <StickKeyBindingActionBase
                       SlopeMode=<%= SlopeMode %>
                       Speed=<%= Speed %>
                       InvertX=<%= InvertX %>
                       InvertY=<%= InvertY %>
                   />
        End Function

#End Region

    End Class

End Namespace
