﻿
Namespace KeyBinding.Serialization

    Public Class DeserializationContext

        Dim _ReferencableConfigurations As New Dictionary(Of Guid, KeyBinding.Configurations.KeyBindingConfiguration)
        Public ReadOnly Property ReferencableConfigurations As Dictionary(Of Guid, KeyBinding.Configurations.KeyBindingConfiguration)
            <DebuggerStepThrough()>
            Get
                Return _ReferencableConfigurations
            End Get
        End Property

        Public Function GetConfigurationByIdentifier(Identifier As Guid) As KeyBinding.Configurations.KeyBindingConfiguration
            Return ReferencableConfigurations(Identifier)
        End Function

        Public Function DeserializeAction(Source As XElement) As KeyBinding.Actions.KeyBindingAction
            For Each i In ThingClasses
                If i.TagName = Source.Name Then
                    Dim Temp As Object
                    Try
                        Temp = i.Constructor.Invoke({Me, Source})
#If CONFIG = "Debug" Then
                    Catch ex As System.Reflection.TargetInvocationException When False
                        Throw
#Else
                    Catch ex As System.Reflection.TargetInvocationException
                        If TypeOf ex.InnerException Is ReferencedConfigurationNotFoundException Then
                            Throw ex.InnerException
                        Else
                            Throw
                        End If
#End If
                    End Try
                    Return DirectCast(Temp, KeyBinding.Actions.KeyBindingAction)
                End If
            Next
            Throw New ArgumentException("Es wurde kein Typ mit einem passenden TagName gefunden.", "Source")
        End Function

        Private Shared ThingClasses As New List(Of DeserializableThing)

        Shared Sub New()
            For Each i In GetType(KeyBinding.Actions.KeyBindingAction).Assembly.GetTypes
                Dim Attributes = i.GetCustomAttributes(GetType(KeyBinding.Serialization.SerializableThingAttribute), False)
                If Attributes.Length <> 0 Then
                    ThingClasses.Add(New DeserializableThing With _
                    {
                        .Type = i, _
                        .TagName = DirectCast(Attributes(0), KeyBinding.Serialization.SerializableThingAttribute).TagName, _
                        .Constructor = i.GetConstructor({GetType(DeserializationContext), GetType(XElement)})
                    })
                End If
            Next
        End Sub

        Private Class DeserializableThing
            Public Type As Type
            Public TagName As String
            Public Constructor As System.Reflection.ConstructorInfo
        End Class

    End Class

End Namespace
