﻿
Namespace Views.DebugGamepadStatesWindow

    Public Class DebugGamepadStatesWindow

        Dim WithEvents ViewModel As DebugGamepadStatesWindowViewModel

        Dim WithEvents PollingTimer As New System.Windows.Threading.DispatcherTimer With {.Interval = TimeSpan.FromMilliseconds(30), .IsEnabled = True}

        Public Sub New()
            InitializeComponent()
            ViewModel = DirectCast(Me.DataContext, DebugGamepadStatesWindowViewModel)
        End Sub

        Private Sub PollingTimer_Tick(sender As Object, e As System.EventArgs) Handles PollingTimer.Tick
            ViewModel.Player1Data = StateToString(XInput.Gamepad.GetState(XInput.PlayerIndex.One))
            ViewModel.Player2Data = StateToString(XInput.Gamepad.GetState(XInput.PlayerIndex.Two))
            ViewModel.Player3Data = StateToString(XInput.Gamepad.GetState(XInput.PlayerIndex.Three))
            ViewModel.Player4Data = StateToString(XInput.Gamepad.GetState(XInput.PlayerIndex.Four))
        End Sub

        Private Shared Function StateToString(State As XInput.IGamepadState) As String
            Dim SB As New IndentStringBuilder
            AppendLine(SB, "IsConnected             ", State.IsConnected)
            AppendLine(SB, "PacketNumber            ", State.PacketNumber)
            AppendLine(SB, "Buttons.A               ", State.GetButtonIsDown(XInput.Button.A))
            AppendLine(SB, "Buttons.B               ", State.GetButtonIsDown(XInput.Button.B))
            AppendLine(SB, "Buttons.X               ", State.GetButtonIsDown(XInput.Button.X))
            AppendLine(SB, "Buttons.Y               ", State.GetButtonIsDown(XInput.Button.Y))
            AppendLine(SB, "Buttons.LeftShoulder    ", State.GetButtonIsDown(XInput.Button.LeftBumper))
            AppendLine(SB, "Buttons.RightShoulder   ", State.GetButtonIsDown(XInput.Button.RightBumper))
            AppendLine(SB, "Buttons.LeftStick       ", State.GetButtonIsDown(XInput.Button.LeftStickButton))
            AppendLine(SB, "Buttons.RightStick      ", State.GetButtonIsDown(XInput.Button.RightStickButton))
            AppendLine(SB, "Buttons.Start           ", State.GetButtonIsDown(XInput.Button.Start))
            AppendLine(SB, "Buttons.Back            ", State.GetButtonIsDown(XInput.Button.Back))
            AppendLine(SB, "DPad.Up                 ", State.GetButtonIsDown(XInput.Button.DPadUp))
            AppendLine(SB, "DPad.Down               ", State.GetButtonIsDown(XInput.Button.DPadDown))
            AppendLine(SB, "DPad.Left               ", State.GetButtonIsDown(XInput.Button.DPadLeft))
            AppendLine(SB, "DPad.Right              ", State.GetButtonIsDown(XInput.Button.DPadRight))
            AppendLine(SB, "ThumbSticks.Left.X      ", State.GetStickValue(XInput.Stick.Left).X)
            AppendLine(SB, "ThumbSticks.Left.Y      ", State.GetStickValue(XInput.Stick.Left).Y)
            AppendLine(SB, "ThumbSticks.Left.Length ", State.GetStickValue(XInput.Stick.Left).Length)
            AppendLine(SB, "ThumbSticks.Right.X     ", State.GetStickValue(XInput.Stick.Right).X)
            AppendLine(SB, "ThumbSticks.Right.Y     ", State.GetStickValue(XInput.Stick.Right).Y)
            AppendLine(SB, "ThumbSticks.Right.Length", State.GetStickValue(XInput.Stick.Right).Length)
            AppendLine(SB, "Triggers.Left           ", State.GetTriggerValue(XInput.Trigger.Left))
            AppendLine(SB, "Triggers.Right          ", State.GetTriggerValue(XInput.Trigger.Right))
            Return SB.ToString
        End Function

        Private Shared Sub AppendLine(SB As IndentStringBuilder, Name As String, Value As Object)
            Dim ValueString As String
            Select Case Value.GetType
                Case GetType(Boolean)
                    ValueString = If(DirectCast(Value, Boolean), "[X]", "[ ]")
                Case Else
                    ValueString = Value.ToString
            End Select
            SB.AppendLine(Name & " {0}", ValueString)
        End Sub

        Protected Overrides Sub OnClosing(e As System.ComponentModel.CancelEventArgs)
            MyBase.OnClosing(e)
            PollingTimer.IsEnabled = False
        End Sub

    End Class

End Namespace
