
Namespace Views.KeyBindingConfigurationDialog

    Public Class KeyBindingConfigurationDialogViewModel
        Inherits ViewModelBase

        Dim _ExistingConfigurations As IEnumerable(Of KeyBinding.Configurations.KeyBindingConfiguration) = Nothing
        Public Property ExistingConfigurations As IEnumerable(Of KeyBinding.Configurations.KeyBindingConfiguration)
            Get
                Return _ExistingConfigurations
            End Get
            Set(value As IEnumerable(Of KeyBinding.Configurations.KeyBindingConfiguration))
                ExtendedChangeIfDifferent(_ExistingConfigurations, value, "ExistingConfigurations")
            End Set
        End Property

        Dim _EditedConfiguration As KeyBindingConfigurationViewModel = Nothing
        Public Property EditedConfiguration As KeyBindingConfigurationViewModel
            Get
                Return _EditedConfiguration
            End Get
            Set(value As KeyBindingConfigurationViewModel)
                ExtendedChangeIfDifferent(_EditedConfiguration, value, "EditedConfiguration")
            End Set
        End Property

        Dim WithEvents _Actions As ViewModelCollection(Of KeyBinding.Configurations.BoundAction, BoundActionViewModel)
        Public Property Actions As ViewModelCollection(Of KeyBinding.Configurations.BoundAction, BoundActionViewModel)
            Get
                Return _Actions
            End Get
            Set(value As ViewModelCollection(Of KeyBinding.Configurations.BoundAction, BoundActionViewModel))
                Dim OldValue = _Actions
                If ChangeIfDifferent(_Actions, value) Then
                    OnExtendedPropertyChanged("Actions", OldValue, value) 'WithEvents Property
                End If
            End Set
        End Property
        Private Sub Actions_ItemCreating(sender As Object, e As ViewModelCollectionItemCreatingEventArgs(Of KeyBinding.Configurations.BoundAction, BoundActionViewModel)) Handles _Actions.ItemCreating
            e.ViewModel = New BoundActionViewModel(e.Item)
            AddHandler e.ViewModel.Remove, AddressOf Action_Remove
            AddHandler e.ViewModel.PropertyChanged, AddressOf Actions_Item_PropertyChanged
        End Sub
        Private Sub Actions_ItemDisposing(sender As Object, e As ViewModelCollectionItemDisposingEventArgs(Of KeyBinding.Configurations.BoundAction, BoundActionViewModel)) Handles _Actions.ItemDisposing
            RemoveHandler e.ViewModel.Remove, AddressOf Action_Remove
            RemoveHandler e.ViewModel.PropertyChanged, AddressOf Actions_Item_PropertyChanged
        End Sub
        Private Sub Actions_ItemsChanged(sender As Object, e As System.EventArgs) Handles _Actions.ItemsChanged
            OnPropertyChanged("CanCloseOk")
        End Sub
        Private Sub Action_Remove(Sender As BoundActionViewModel)
            If MessageBox.Show("Soll diese Aktion wirklich gelscht werden? Dies kann nur durch Abbrechen des Dialogs rckgngig gemacht werden", "Wirklich lschen?", MessageBoxButton.YesNo, MessageBoxImage.Warning) = MessageBoxResult.Yes Then
                EditedConfiguration.Target.Actions.Remove(Sender.Target)
            End If
        End Sub
        Private Sub Actions_Item_PropertyChanged(sender As Object, e As PropertyChangedEventArgs)
            Select Case e.PropertyName
                Case "IsValid"
                    OnPropertyChanged("CanCloseOk")
            End Select
        End Sub

        <Dependency("Actions")>
        Public ReadOnly Property CanCloseOk As Boolean
            Get
                If Actions Is Nothing Then
                    Return True
                End If
                Return Actions.ViewModels.All(Function(i) i.IsValid)
            End Get
        End Property

        Public Sub New()
            MyBase.New(True)
            If IsInDesignMode Then
                SetEditedData(DebugData.DebugConfigurations.Skip(1), DebugData.DebugConfigurations(0))
            End If
        End Sub

        Public Sub SetEditedData(NewExistingConfigurations As IEnumerable(Of KeyBinding.Configurations.KeyBindingConfiguration), NewEditedConfiguration As KeyBinding.Configurations.KeyBindingConfiguration)
            ExistingConfigurations = NewExistingConfigurations
            EditedConfiguration = New KeyBindingConfigurationViewModel(NewEditedConfiguration)
            Actions = New ViewModelCollection(Of KeyBinding.Configurations.BoundAction, BoundActionViewModel)(New ObservableCollectionSource(Of KeyBinding.Configurations.BoundAction)(EditedConfiguration.Target.Actions))
            Actions.SynchronizeWithList(EditedConfiguration.Target.Actions)
        End Sub


    End Class

End Namespace
