﻿
Namespace Views.KeyBindingConfigurationDialog.ViewModels.Button

    Public Class MultiActionSubActionViewModel
        Inherits ViewModelBase(Of KeyBinding.Actions.Button.MultiActionSubAction)

        Public Event Remove(Sender As MultiActionSubActionViewModel)
        Public ReadOnly Property RemoveCommand As DelegateCommand
            Get
                Static Temp As New DelegateCommand(Sub() RaiseEvent Remove(Me))
                Return Temp
            End Get
        End Property

        Public Event MoveUp(Sender As MultiActionSubActionViewModel)
        Public ReadOnly Property MoveUpCommand As DelegateCommand
            Get
                Static Temp As New DelegateCommand(Sub() RaiseEvent MoveUp(Me))
                Return Temp
            End Get
        End Property

        Public Event MoveDown(Sender As MultiActionSubActionViewModel)
        Public ReadOnly Property MoveDownCommand As DelegateCommand
            Get
                Static Temp As New DelegateCommand(Sub() RaiseEvent MoveDown(Me))
                Return Temp
            End Get
        End Property

        Public ReadOnly Property AvailableActionTypes As IEnumerable(Of Type)
            Get
                Return KeyBinding.Actions.KeyBindingAction.SubTypesOfInputKind(GamepadData.GamepadInputKind.Button)
            End Get
        End Property

        Public Property SelectedActionType As Type
            Get
                If Target.Action Is Nothing Then
                    Return Nothing
                End If
                Return Target.Action.GetType
            End Get
            Set(value As Type)
                If Target.Action IsNot Nothing AndAlso Target.Action.GetType = value Then
                    Return
                End If
                Target.Action = value.RunDefaultConstructor(Of KeyBinding.Actions.Button.ButtonKeyBindingActionBase)()
            End Set
        End Property

        Dim _SubViewModel As ViewModels.KeyBindingConfigurationDialogSubViewModel = Nothing
        Public Property SubViewModel As ViewModels.KeyBindingConfigurationDialogSubViewModel
            Get
                Return _SubViewModel
            End Get
            Private Set(value As ViewModels.KeyBindingConfigurationDialogSubViewModel)
                If _SubViewModel IsNot value Then
                    Dim OldValue = _SubViewModel
                    If _SubViewModel IsNot Nothing Then
                        RemoveHandler _SubViewModel.ExtendedPropertyChanged, AddressOf SubViewModel_ExtendedPropertyChanged
                    End If
                    _SubViewModel = value
                    If _SubViewModel IsNot Nothing Then
                        AddHandler _SubViewModel.ExtendedPropertyChanged, AddressOf SubViewModel_ExtendedPropertyChanged
                    End If
                    OnExtendedPropertyChanged("SubViewModel", OldValue, value)
                End If
            End Set
        End Property

        <Dependency("SubViewModel")>
        Public ReadOnly Property IsValid As Boolean
            Get
                Return Target.Action IsNot Nothing AndAlso SubViewModel IsNot Nothing AndAlso SubViewModel.IsValid
            End Get
        End Property

        Public Sub New(NewTarget As KeyBinding.Actions.Button.MultiActionSubAction)
            MyBase.New(NewTarget)
            AddExternalPropertyDependency("IsValid", Target, "Action")
            AddExternalPropertyChangedHandler(Target, {"Action"}, AddressOf Target_PropertyChanged)
            UpdateSubViewModel()
        End Sub

        Private Sub Target_PropertyChanged(sender As Object, e As PropertyChangedEventArgs)
            Select Case e.PropertyName
                Case "Action"
                    UpdateSubViewModel()
            End Select
        End Sub

        Private Sub SubViewModel_ExtendedPropertyChanged(sender As Object, e As ExtendedPropertyChangedEventArgs)
            Select Case e.PropertyName
                Case "IsValid"
                    OnPropertyChanged("IsValid")
            End Select
        End Sub

        Private Sub UpdateSubViewModel()
            If Target.Action Is Nothing Then
                SubViewModel = Nothing
            Else
                SubViewModel = ViewModels.KeyBindingConfigurationDialogSubViewModel.GetViewModelFor(Target.Action)
            End If
        End Sub

    End Class

End Namespace
