﻿
Namespace Views.KeyBindingConfigurationDialog.ViewModels.Button

    <ViewModelForAttribute(GetType(KeyBinding.Actions.Button.SequenceAction))>
    Public Class SequenceAction
        Inherits KeyBindingConfigurationDialogSubViewModel

        Public ReadOnly Property AddNewUpActionCommand As DelegateCommand
            Get
                Static Temp As New DelegateCommand(Sub() CastedTarget.Actions.Add(New KeyBinding.Actions.Button.SequenceActionSubAction With {.Action = Nothing}))
                Return Temp
            End Get
        End Property

        Public Overrides ReadOnly Property IsValid As Boolean
            Get
                Return CastedTarget.Actions.Count <> 0 AndAlso Actions.ViewModels.All(Function(i) i.IsValid)
            End Get
        End Property

        Dim WithEvents _Actions As ViewModelCollection(Of KeyBinding.Actions.Button.SequenceActionSubAction, SequenceActionSubActionViewModel)
        Public Property Actions As ViewModelCollection(Of KeyBinding.Actions.Button.SequenceActionSubAction, SequenceActionSubActionViewModel)
            Get
                Return _Actions
            End Get
            Set(value As ViewModelCollection(Of KeyBinding.Actions.Button.SequenceActionSubAction, SequenceActionSubActionViewModel))
                Dim OldValue = _Actions
                If ChangeIfDifferent(_Actions, value) Then
                    OnExtendedPropertyChanged("Actions", OldValue, value) 'WithEvents Property
                End If
            End Set
        End Property
        Private Sub Actions_ItemCreating(sender As Object, e As ViewModelCollectionItemCreatingEventArgs(Of KeyBinding.Actions.Button.SequenceActionSubAction, SequenceActionSubActionViewModel)) Handles _Actions.ItemCreating
            e.ViewModel = New SequenceActionSubActionViewModel(e.Item)
            AddHandler e.ViewModel.Remove, AddressOf Action_Remove
            AddHandler e.ViewModel.PropertyChanged, AddressOf Action_PropertyChanged
            AddHandler e.ViewModel.MoveUp, AddressOf Action_MoveUp
            AddHandler e.ViewModel.MoveDown, AddressOf Action_MoveDown
        End Sub
        Private Sub Actions_ItemDisposing(sender As Object, e As ViewModelCollectionItemDisposingEventArgs(Of KeyBinding.Actions.Button.SequenceActionSubAction, SequenceActionSubActionViewModel)) Handles _Actions.ItemDisposing
            RemoveHandler e.ViewModel.Remove, AddressOf Action_Remove
            RemoveHandler e.ViewModel.PropertyChanged, AddressOf Action_PropertyChanged
            RemoveHandler e.ViewModel.MoveUp, AddressOf Action_MoveUp
            RemoveHandler e.ViewModel.MoveDown, AddressOf Action_MoveDown
        End Sub
        Private Sub Action_Remove(Sender As SequenceActionSubActionViewModel)
            CastedTarget.Actions.Remove(Sender.Target)
        End Sub
        Private Sub Action_MoveUp(Sender As SequenceActionSubActionViewModel)
            Dim Index = CastedTarget.Actions.IndexOf(Sender.Target)
            If Index > 0 Then
                CastedTarget.Actions.Move(Index, Index - 1)
            End If
        End Sub
        Private Sub Action_MoveDown(Sender As SequenceActionSubActionViewModel)
            Dim Index = CastedTarget.Actions.IndexOf(Sender.Target)
            If Index < CastedTarget.Actions.Count - 1 Then
                CastedTarget.Actions.Move(Index, Index + 1)
            End If
        End Sub

        Dim _CastedTarget As KeyBinding.Actions.Button.SequenceAction
        Public ReadOnly Property CastedTarget As KeyBinding.Actions.Button.SequenceAction
            Get
                Return _CastedTarget
            End Get
        End Property

        Public Sub New(NewTarget As KeyBinding.Actions.Button.SequenceAction)
            MyBase.New(NewTarget)
            _CastedTarget = NewTarget
            AddExternalPropertyDependency("IsValid", _CastedTarget.Actions, "Count")
            Actions = New ViewModelCollection(Of KeyBinding.Actions.Button.SequenceActionSubAction, SequenceActionSubActionViewModel)(New ObservableCollectionSource(Of KeyBinding.Actions.Button.SequenceActionSubAction)(_CastedTarget.Actions))
            Actions.SynchronizeWithList(CastedTarget.Actions)
        End Sub

        Private Sub Action_PropertyChanged(sender As Object, e As PropertyChangedEventArgs)
            Select Case e.PropertyName
                Case "IsValid"
                    OnPropertyChanged("IsValid")
            End Select
        End Sub

    End Class

End Namespace
