﻿
Namespace Views.MainWindow

    Class MainWindow

        Dim WithEvents _Configurations As New TypedObservableCollection(Of KeyBinding.Configurations.KeyBindingConfiguration)
        Public ReadOnly Property Configurations As TypedObservableCollection(Of KeyBinding.Configurations.KeyBindingConfiguration)
            Get
                Return _Configurations
            End Get
        End Property
        Private Sub Configurations_ItemAdded(sender As Object, e As ItemEventArgs(Of KeyBinding.Configurations.KeyBindingConfiguration)) Handles _Configurations.ItemAdded
            AddHandler e.Item.PublicAction, AddressOf Configuration_PublicAction
        End Sub
        Private Sub Configurations_ItemRemoved(sender As Object, e As ItemEventArgs(Of KeyBinding.Configurations.KeyBindingConfiguration)) Handles _Configurations.ItemRemoved
            RemoveHandler e.Item.PublicAction, AddressOf Configuration_PublicAction
        End Sub

        Dim WithEvents _Gamepads As New TypedObservableCollection(Of GamepadData.Gamepad)
        Public ReadOnly Property Gamepads As TypedObservableCollection(Of GamepadData.Gamepad)
            Get
                Return _Gamepads
            End Get
        End Property
        Private Sub Gamepads_ItemAdded(sender As Object, e As ItemEventArgs(Of GamepadData.Gamepad)) Handles _Gamepads.ItemAdded
            AddHandler e.Item.PropertyChanged, AddressOf Gamepsads_Item_PropertyChanged
        End Sub
        Private Sub Gamepads_ItemRemoved(sender As Object, e As ItemEventArgs(Of GamepadData.Gamepad)) Handles _Gamepads.ItemRemoved
            RemoveHandler e.Item.PropertyChanged, AddressOf Gamepsads_Item_PropertyChanged
        End Sub
        Private Sub Gamepads_TypedCollectionChanged(sender As Object, e As EventArgs) Handles _Gamepads.TypedCollectionChanged
            UpdateTimerEnabledStates()
        End Sub
        Private Sub Gamepsads_Item_PropertyChanged(sender As Object, e As PropertyChangedEventArgs)
            Select Case e.PropertyName
                Case "Configuration", "IsActive"
                    UpdateTimerEnabledStates()
            End Select
        End Sub

        Dim WithEvents ViewModel As MainWindowViewModel

        Public Sub New()
            InitializeComponent()
            ViewModel = DirectCast(Me.DataContext, MainWindowViewModel)

            Gamepads.Add(New GamepadData.Gamepad(GamepadData.PlayerNumber.One))
            Gamepads.Add(New GamepadData.Gamepad(GamepadData.PlayerNumber.Two))
            Gamepads.Add(New GamepadData.Gamepad(GamepadData.PlayerNumber.Three))
            Gamepads.Add(New GamepadData.Gamepad(GamepadData.PlayerNumber.Four))
            LoadSettings()

            ViewModel.Configurations = New ViewModelCollection(Of KeyBinding.Configurations.KeyBindingConfiguration, KeyBindingConfigurationViewModel)(New ObservableCollectionSource(Of KeyBinding.Configurations.KeyBindingConfiguration)(Configurations))
            ViewModel.Configurations.SynchronizeWithList(Configurations)
            If Configurations.Count <> 0 Then
                ViewModel.SelectedConfiguration = ViewModel.Configurations.FindViewModel(Configurations(0))
            End If

            ViewModel.Gamepads = New ViewModelCollection(Of GamepadData.Gamepad, GamepadViewModel)(New ObservableCollectionSource(Of GamepadData.Gamepad)(Gamepads))
            ViewModel.Gamepads.SynchronizeWithList(Gamepads)
            If Gamepads.Count <> 0 Then
                ViewModel.SelectedGamepad = ViewModel.Gamepads.FindViewModel(Gamepads(0))
            End If
            UpdateTimerEnabledStates()
        End Sub

        Protected Overrides Sub OnClosing(e As System.ComponentModel.CancelEventArgs)
            SaveSettings()
            MyBase.OnClosing(e)
        End Sub

#Region "Edit Configurations"

        Private Sub ViewModel_ConfigurationRemove(Configuration As KeyBindingConfigurationViewModel) Handles ViewModel.ConfigurationRemove
            If MessageBox.Show("Soll diese Konfiguration wirklich gelöscht werden? Dies kann nicht rückgängig gemacht werden!", "Wirklich löschen?", MessageBoxButton.YesNo, MessageBoxImage.Warning) = MessageBoxResult.Yes Then
                Configurations.Remove(Configuration.Target)
                SaveSettings()
            End If
        End Sub

        Private Sub ViewModel_ConfigurationMoveUp(Configuration As KeyBindingConfigurationViewModel) Handles ViewModel.ConfigurationMoveUp
            Dim Index = Configurations.IndexOf(Configuration.Target)
            If Index > 0 Then
                Configurations.Move(Index, Index - 1)
                SaveSettings()
            End If
        End Sub

        Private Sub ViewModel_ConfigurationMoveDown(Configuration As KeyBindingConfigurationViewModel) Handles ViewModel.ConfigurationMoveDown
            Dim Index = Configurations.IndexOf(Configuration.Target)
            If Index < Configurations.Count - 1 Then
                Configurations.Move(Index, Index + 1)
                SaveSettings()
            End If
        End Sub

        Private Sub ViewModel_ConfigurationEdit(Configuration As KeyBindingConfigurationViewModel) Handles ViewModel.ConfigurationEdit
            Dim Dlg As New Views.KeyBindingConfigurationDialog.KeyBindingConfigurationDialog(Configurations.ExceptItem(Configuration.Target), Configuration.Target.Clone) With {.Owner = Me}
            If Dlg.ShowDialog() Then
                Dim OldConfiguration = Configuration.Target
                Dim NewConfiguration = Dlg.EditedConfiguration

                For Each i In Gamepads
                    If i.Configuration Is OldConfiguration Then
                        i.Configuration = NewConfiguration
                    End If
                Next

                'Through DataBinding, this code sets all gamepads' Configuration properties to Nothing where the previous value was the affected configuration.
                Configurations(Configurations.IndexOf(Configuration.Target)) = NewConfiguration

                For Each i In Configurations
                    For Each j In i.Actions
                        j.Action.ReplaceReferencedConfiguration(OldConfiguration, NewConfiguration)
                    Next
                Next

                SaveSettings()
            End If
        End Sub

        Private Sub AddNewConfiguration(sender As System.Object, e As System.Windows.RoutedEventArgs)
            Configurations.Add(New KeyBinding.Configurations.KeyBindingConfiguration With {.Name = "Neue Konfiguration"})
            SaveSettings()
        End Sub

#End Region

#Region "Public Actions"

        Private Sub Configuration_PublicAction(sender As Object, e As KeyBinding.Actions.KeyBindingPublicActionEventArgs)
            Select Case e.GetType
                Case GetType(KeyBinding.Actions.Button.ChangeConfigurationAction.ChangeConfigurationRequestedEventArgs)
                    Dim CastedArgs = DirectCast(e, KeyBinding.Actions.Button.ChangeConfigurationAction.ChangeConfigurationRequestedEventArgs)
                    e.SenderGamepad.Configuration = CastedArgs.NewConfiguration
                    SaveSettings()
                Case GetType(KeyBinding.Actions.Button.MultiAction.MultiActionInvokedEventArgs)
                    Dim CastedAction = DirectCast(e.SenderAction, KeyBinding.Actions.Button.MultiAction)
                    Dim CursorPosition = System.Windows.Forms.Cursor.Position
                    OpenMultiActionSelectorWindow(e.SenderGamepad, CastedAction, New Point(CursorPosition.X, CursorPosition.Y))
            End Select
        End Sub

        Private Sub OpenMultiActionSelectorWindow(SenderGamepad As GamepadData.Gamepad, Action As KeyBinding.Actions.Button.MultiAction, CenterOnScreen As Point)
            Dim SelectorWindow = New Views.MultiActionSelectorWindow.MultiActionSelectorWindow(Action, CenterOnScreen) With {.Owner = Me}
            AddHandler SelectorWindow.Closed, Sub()
                                                  If SenderGamepad.TemporaryInterceptor Is SelectorWindow Then
                                                      SenderGamepad.TemporaryInterceptor = Nothing
                                                  End If
                                              End Sub
            SenderGamepad.TemporaryInterceptor = SelectorWindow
            SelectorWindow.Show()
        End Sub

#End Region

#Region "Debug Window"

        Dim DebugWindowInstance As Views.DebugGamepadStatesWindow.DebugGamepadStatesWindow = Nothing

        Private Sub OpenDebugWindow() Handles ViewModel.OpenDebugWindow
            If DebugWindowInstance Is Nothing Then
                DebugWindowInstance = New Views.DebugGamepadStatesWindow.DebugGamepadStatesWindow With {.Owner = Me}
                AddHandler DebugWindowInstance.Closed, AddressOf DebugWindowInstance_Closed
                DebugWindowInstance.Show()
            Else
                DebugWindowInstance.Focus()
            End If
        End Sub

        Private Sub DebugWindowInstance_Closed(sender As Object, e As EventArgs)
            DebugWindowInstance = Nothing
        End Sub

#End Region

#Region "Timer Actions"

        Dim WithEvents GamepadPollTimer As New System.Windows.Threading.DispatcherTimer With {.Interval = TimeSpan.FromMilliseconds(20), .IsEnabled = False}
        Dim WithEvents ContinuousActionTimer As New System.Windows.Threading.DispatcherTimer With {.Interval = TimeSpan.FromMilliseconds(5), .IsEnabled = False}

        Dim TickTimestamp As DateTime
        Dim HasTickTimestamp As Boolean
        
        Private Sub UpdateTimerEnabledStates()
            GamepadPollTimer.IsEnabled = Gamepads.Any(Function(i) i.Configuration IsNot Nothing AndAlso i.IsActive)
            If ContinuousActionTimer.IsEnabled <> GamepadPollTimer.IsEnabled Then
                ContinuousActionTimer.IsEnabled = GamepadPollTimer.IsEnabled
                HasTickTimestamp = False
            End If
        End Sub

        Private Sub GamepadPollTimer_Tick(sender As Object, e As System.EventArgs) Handles GamepadPollTimer.Tick
            For Each i In Gamepads
                i.UpdateStateAndApplyConfigurationForButtonActions()
            Next
        End Sub

        Private Sub ContinuousActionTimer_Tick(sender As Object, e As System.EventArgs) Handles ContinuousActionTimer.Tick
            Dim NextTickTimestamp = DateTime.Now
            Dim SecondsSinceLastTick As Double
            If HasTickTimestamp Then
                SecondsSinceLastTick = (NextTickTimestamp - TickTimestamp).TotalSeconds
            Else
                SecondsSinceLastTick = ContinuousActionTimer.Interval.TotalSeconds
            End If
            TickTimestamp = NextTickTimestamp
            HasTickTimestamp = True
            For Each i In Gamepads
                If i.IsActive AndAlso i.Configuration IsNot Nothing Then
                    For Each j In i.Configuration.Actions
                        If TypeOf j.Action Is KeyBinding.Actions.IContinuousAction Then
                            DirectCast(j.Action, KeyBinding.Actions.IContinuousAction).ApplyAction(i, SecondsSinceLastTick)
                        End If
                    Next
                End If
            Next
        End Sub

#End Region

#Region "Settings"

        Public ReadOnly Property SettingsDirectoryPath As String
            Get
                Static Temp As String = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), "GamepadControl")
                Return Temp
            End Get
        End Property

        Public ReadOnly Property SettingsFilePath As String
            Get
                Static Temp As String = System.IO.Path.Combine(SettingsDirectoryPath, "Settings.xml")
                Return Temp
            End Get
        End Property

        Private Sub SaveSettings()
            If Not System.IO.Directory.Exists(SettingsDirectoryPath) Then
                System.IO.Directory.CreateDirectory(SettingsDirectoryPath)
            End If
            Dim Context As New KeyBinding.Serialization.SerializationContext
            For Each i In Configurations
                Context.ReferencableConfigurations.Add(i, Guid.NewGuid)
            Next
            Dim Xml = <Settings>
                          <Configurations>
                              <%= Context.ReferencableConfigurations.Select(Function(i) <Configuration
                                                                                            Name=<%= i.Key.Name %>
                                                                                            Identifier=<%= i.Value %>
                                                                                            >
                                                                                            <Actions>
                                                                                                <%= i.Key.Actions.Select(Function(j) <BoundAction
                                                                                                                                         Input=<%= j.Input %>
                                                                                                                                         >
                                                                                                                                         <%= j.Action.Serialize(Context) %>
                                                                                                                                     </BoundAction>) %>
                                                                                            </Actions>
                                                                                        </Configuration>) %>
                          </Configurations>
                          <Gamepads>
                              <%= Gamepads.Select(Function(i) <Gamepad
                                                                  PlayerNumber=<%= i.PlayerNumber %>
                                                                  IsActive=<%= i.IsActive %>
                                                                  Configuration=<%= If(i.Configuration Is Nothing, Nothing, DirectCast(Context.GetConfigurationIdentifier(i.Configuration), Object)) %>
                                                              />) %>
                          </Gamepads>
                      </Settings>
            Xml.Save(SettingsFilePath)
        End Sub

        Private Sub LoadSettings()
            If Not System.IO.File.Exists(SettingsFilePath) Then
                Return
            End If
            Dim ConfigurationsToProcess As New List(Of Tuple(Of KeyBinding.Configurations.KeyBindingConfiguration, XElement))
            Dim Xml = XElement.Load(SettingsFilePath)
            Dim Context As New KeyBinding.Serialization.DeserializationContext
            For Each i In Xml.<Configurations>.Elements
                Dim NewConfiguration As New KeyBinding.Configurations.KeyBindingConfiguration With {.Name = i.GetAttributeString("Name")}
                ConfigurationsToProcess.Add(Tuple.Create(NewConfiguration, i))
                Context.ReferencableConfigurations.Add(i.GetAttributeGuid("Identifier"), NewConfiguration)
            Next
            For Each i In ConfigurationsToProcess
                For Each j In i.Item2.<Actions>.Elements
                    Dim BoundAction As New KeyBinding.Configurations.BoundAction
                    BoundAction.Input = j.GetAttributeEnum(Of GamepadData.GamepadInput)("Input")
                    BoundAction.Action = Context.DeserializeAction(j.Elements.Single)
                    i.Item1.Actions.Add(BoundAction)
                Next
            Next
            Configurations.Clear()
            Configurations.AddRange(ConfigurationsToProcess.Select(Function(i) i.Item1))
            For Each i In Xml.<Gamepads>.Elements
                Dim PlayerNumber = i.GetAttributeEnum(Of GamepadData.PlayerNumber)("PlayerNumber")
                Dim IsActive = i.GetAttributeBoolean("IsActive")
                Dim ConfigurationIdentifier = i.TryGetAttributeGuid("Configuration")
                Dim Gamepad = Gamepads.Single(Function(j) j.PlayerNumber = PlayerNumber)
                If ConfigurationIdentifier Is Nothing Then
                    Gamepad.Configuration = Nothing
                Else
                    Gamepad.Configuration = Context.GetConfigurationByIdentifier(ConfigurationIdentifier.Value)
                End If
                Gamepad.IsActive = IsActive
            Next
        End Sub

#End Region

    End Class

End Namespace
