﻿
Namespace Views.MainWindow

    Class MainWindowViewModel
        Inherits ViewModelBase

        Public Event OpenDebugWindow()
        Public ReadOnly Property OpenDebugWindowCommand As DelegateCommand
            Get
                Static Temp As New DelegateCommand(Sub() RaiseEvent OpenDebugWindow())
                Return Temp
            End Get
        End Property

        Public Event ConfigurationRemove(Configuration As KeyBindingConfigurationViewModel)
        Protected Overridable Sub OnConfigurationRemove(Configuration As KeyBindingConfigurationViewModel)
            RaiseEvent ConfigurationRemove(Configuration)
        End Sub

        Public Event ConfigurationMoveUp(Configuration As KeyBindingConfigurationViewModel)
        Protected Overridable Sub OnConfigurationMoveUp(Configuration As KeyBindingConfigurationViewModel)
            RaiseEvent ConfigurationMoveUp(Configuration)
        End Sub

        Public Event ConfigurationMoveDown(Configuration As KeyBindingConfigurationViewModel)
        Protected Overridable Sub OnConfigurationMoveDown(Configuration As KeyBindingConfigurationViewModel)
            RaiseEvent ConfigurationMoveDown(Configuration)
        End Sub

        Public Event ConfigurationEdit(Configuration As KeyBindingConfigurationViewModel)
        Protected Overridable Sub OnConfigurationEdit(Configuration As KeyBindingConfigurationViewModel)
            RaiseEvent ConfigurationEdit(Configuration)
        End Sub

        Dim WithEvents _Configurations As ViewModelCollection(Of KeyBinding.Configurations.KeyBindingConfiguration, KeyBindingConfigurationViewModel)
        Public Property Configurations As ViewModelCollection(Of KeyBinding.Configurations.KeyBindingConfiguration, KeyBindingConfigurationViewModel)
            Get
                Return _Configurations
            End Get
            Set(value As ViewModelCollection(Of KeyBinding.Configurations.KeyBindingConfiguration, KeyBindingConfigurationViewModel))
                Dim OldValue = _Configurations
                If ChangeIfDifferent(_Configurations, value) Then
                    OnExtendedPropertyChanged("Configurations", OldValue, value) 'WithEvents Property
                End If
            End Set
        End Property
        Private Sub Configurations_ItemCreating(sender As Object, e As ViewModelCollectionItemCreatingEventArgs(Of KeyBinding.Configurations.KeyBindingConfiguration, KeyBindingConfigurationViewModel)) Handles _Configurations.ItemCreating
            e.ViewModel = New KeyBindingConfigurationViewModel(e.Item)
            AddHandler e.ViewModel.Remove, AddressOf OnConfigurationRemove
            AddHandler e.ViewModel.MoveUp, AddressOf OnConfigurationMoveUp
            AddHandler e.ViewModel.MoveDown, AddressOf OnConfigurationMoveDown
            AddHandler e.ViewModel.Edit, AddressOf OnConfigurationEdit
        End Sub
        Private Sub Configurations_ItemDisposing(sender As Object, e As ViewModelCollectionItemDisposingEventArgs(Of KeyBinding.Configurations.KeyBindingConfiguration, KeyBindingConfigurationViewModel)) Handles _Configurations.ItemDisposing
            RemoveHandler e.ViewModel.Remove, AddressOf OnConfigurationRemove
            RemoveHandler e.ViewModel.MoveUp, AddressOf OnConfigurationMoveUp
            RemoveHandler e.ViewModel.MoveDown, AddressOf OnConfigurationMoveDown
            RemoveHandler e.ViewModel.Edit, AddressOf OnConfigurationEdit
        End Sub

        <Dependency("Configurations")>
        Public ReadOnly Property Configurations_ViewModels_Targets As ObservableSelectorCollection(Of KeyBindingConfigurationViewModel, ReadOnlyObservableCollection(Of KeyBindingConfigurationViewModel), KeyBinding.Configurations.KeyBindingConfiguration)
            Get
                Return New ObservableSelectorCollection(Of KeyBindingConfigurationViewModel, ReadOnlyObservableCollection(Of KeyBindingConfigurationViewModel), KeyBinding.Configurations.KeyBindingConfiguration) _
                       (Configurations.ViewModels, Function(i) i.Target)
            End Get
        End Property

        Dim _SelectedConfiguration As KeyBindingConfigurationViewModel = Nothing
        Public Property SelectedConfiguration As KeyBindingConfigurationViewModel
            Get
                Return _SelectedConfiguration
            End Get
            Set(value As KeyBindingConfigurationViewModel)
                ExtendedChangeIfDifferent(_SelectedConfiguration, value, "SelectedConfiguration")
            End Set
        End Property

        Dim WithEvents _Gamepads As ViewModelCollection(Of GamepadData.Gamepad, GamepadViewModel)
        Public Property Gamepads As ViewModelCollection(Of GamepadData.Gamepad, GamepadViewModel)
            Get
                Return _Gamepads
            End Get
            Set(value As ViewModelCollection(Of GamepadData.Gamepad, GamepadViewModel))
                Dim OldValue = _Gamepads
                If ChangeIfDifferent(_Gamepads, value) Then
                    OnExtendedPropertyChanged("Gamepads", OldValue, value) 'WithEvents Property
                End If
            End Set
        End Property
        Private Sub Gamepads_ItemCreating(sender As Object, e As ViewModelCollectionItemCreatingEventArgs(Of GamepadData.Gamepad, GamepadViewModel)) Handles _Gamepads.ItemCreating
            e.ViewModel = New GamepadViewModel(e.Item)
        End Sub
        Private Sub Gamepads_ItemDisposing(sender As Object, e As ViewModelCollectionItemDisposingEventArgs(Of GamepadData.Gamepad, GamepadViewModel)) Handles _Gamepads.ItemDisposing

        End Sub

        Dim _SelectedGamepad As GamepadViewModel = Nothing
        Public Property SelectedGamepad As GamepadViewModel
            Get
                Return _SelectedGamepad
            End Get
            Set(value As GamepadViewModel)
                ExtendedChangeIfDifferent(_SelectedGamepad, value, "SelectedGamepad")
            End Set
        End Property

        Public Sub New()
            MyBase.New(True)
            If IsInDesignMode Then
                Dim TempConfigurations As New ObservableCollection(Of KeyBinding.Configurations.KeyBindingConfiguration)
                _Configurations = New ViewModelCollection(Of KeyBinding.Configurations.KeyBindingConfiguration, KeyBindingConfigurationViewModel)(New ObservableCollectionSource(Of KeyBinding.Configurations.KeyBindingConfiguration)(TempConfigurations))
                TempConfigurations.AddRange(DebugData.DebugConfigurations)
                SelectedConfiguration = Configurations.FindViewModel(TempConfigurations(0))

                Dim TempGamepads As New ObservableCollection(Of GamepadData.Gamepad)
                _Gamepads = New ViewModelCollection(Of GamepadData.Gamepad, GamepadViewModel)(New ObservableCollectionSource(Of GamepadData.Gamepad)(TempGamepads))
                TempGamepads.AddRange(DebugData.DebugGamepads)
                SelectedGamepad = Gamepads.FindViewModel(TempGamepads(0))
            End If
        End Sub

    End Class

End Namespace
