﻿
Imports Direction = GamepadControl.KeyBinding.Actions.Direction

Namespace Views.MultiActionSelectorWindow

    Public Class MultiActionSelectorWindow
        Implements GamepadData.ITemporaryGamepadStateInterceptor

        Dim ViewModel As MultiActionSelectorWindowViewModel
        Dim CenterOnScreen As Point
        Dim LeftStickDirection As Direction? = Nothing
        Dim RightStickDirection As Direction? = Nothing
        Dim DPadStickDirection As Direction? = Nothing

        Dim AButtonIsDown As Boolean = False
        Dim BButtonIsDown As Boolean = False

        Public Sub New()
            Me.New(DebugData.DebugMultiAction, New Point(500, 500))
            ViewModel.UpActions.ViewModels(0).IsSelected = True
        End Sub

        Public Sub New(NewAction As KeyBinding.Actions.Button.MultiAction, NewCenterOnScreen As Point)
            CenterOnScreen = NewCenterOnScreen
            InitializeComponent()
            ViewModel = DirectCast(Me.DataContext, MultiActionSelectorWindowViewModel)
            ViewModel.SetData(NewAction)
        End Sub

        Private Sub MultiActionSelectorWindow_Loaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles Me.Loaded
            Dim CenterControlPosition = CenterControl.TranslatePoint(New Point(0, 0), Me)
            Dim WindowPosition = CenterOnScreen - CenterControlPosition
            Me.Left = WindowPosition.X
            Me.Top = WindowPosition.Y
        End Sub

        Public Sub Intercept(SenderGamepad As GamepadData.Gamepad, State As XInput.IGamepadState) Implements GamepadData.ITemporaryGamepadStateInterceptor.Intercept
            If ViewModel.CurrentSelectionIndex <> -1 AndAlso State.GetButtonIsDown(XInput.Button.A) Then
                AButtonIsDown = True
            Else
                If AButtonIsDown Then
                    InvokeSelectedAction(SenderGamepad)
                    Me.Close()
                    Return
                End If
            End If
            If State.GetButtonIsDown(XInput.Button.B) Then
                BButtonIsDown = True
            Else
                If BButtonIsDown Then
                    Me.Close()
                End If
            End If
            CompareSetAndSelect(LeftStickDirection, DirectionFromStick(State.GetStickValue(XInput.Stick.Left)))
            CompareSetAndSelect(RightStickDirection, DirectionFromStick(State.GetStickValue(XInput.Stick.Right)))
            CompareSetAndSelect(DPadStickDirection, DirectionFromDPad(State))
        End Sub

        Private Sub CompareSetAndSelect(ByRef CurrentDirection As Direction?, NewDirection As Direction?)
            If CurrentDirection.NullableEquals(NewDirection) Then
                Return
            End If
            CurrentDirection = NewDirection
            If NewDirection.HasValue Then
                SelectInDirection(NewDirection.Value)
            End If
        End Sub

        Private Sub SelectInDirection(Direction As Direction)
            If ViewModel.CurrentSelectionIndex = -1 Then
                ViewModel.CurrentSelectionDirection = Direction
                ViewModel.CurrentSelectionIndex = 0
            ElseIf ViewModel.CurrentSelectionDirection = Direction Then
                If IsValidIndexForThisDirection(Direction, ViewModel.CurrentSelectionIndex + 1) Then
                    ViewModel.CurrentSelectionIndex += 1
                End If
            ElseIf ViewModel.CurrentSelectionDirection = Direction.GetFieldAttribute(Of KeyBinding.Actions.OppositeDirectionAttribute)().OppositeDirection Then
                ViewModel.CurrentSelectionIndex -= 1
            ElseIf IsValidIndexForThisDirection(Direction, 0) Then
                ViewModel.CurrentSelectionIndex = 0
                ViewModel.CurrentSelectionDirection = Direction
            End If
        End Sub

        Private Function IsValidIndexForThisDirection(ThisDirection As Direction, Index As Integer) As Boolean
            Return Index < ViewModel.ActionsByDirection(ThisDirection).Count
        End Function

        Private Sub InvokeSelectedAction(SenderGamepad As GamepadData.Gamepad)
            If ViewModel.CurrentSelectionIndex <> -1 Then
                Dim Action = ViewModel.ActionsByDirection(ViewModel.CurrentSelectionDirection)(ViewModel.CurrentSelectionIndex).Target.Action
                Action.UpdateIsButtonDown(True, SenderGamepad)
                Action.UpdateIsButtonDown(False, SenderGamepad)
            End If
        End Sub

        Private Function DirectionFromStick(StickValue As Vector2D) As Direction?
            If StickValue.Length < 0.5 Then
                Return Nothing
            End If
            Dim Angle = Math.Atan2(-StickValue.Y, -StickValue.X) + Math.PI
            Const QuarterRotation = Math.PI * 2.0 / 4.0
            Const HalfQuarterRotation = QuarterRotation / 2.0
            Const TopRight = HalfQuarterRotation
            Const TopLeft = TopRight + QuarterRotation
            Const BottomLeft = TopLeft + QuarterRotation
            Const BottomRight = BottomLeft + QuarterRotation
            If Angle < TopRight Then
                Return Direction.Right
            ElseIf Angle >= TopRight AndAlso Angle < TopLeft Then
                Return Direction.Up
            ElseIf Angle >= TopLeft AndAlso Angle < BottomLeft Then
                Return Direction.Left
            ElseIf Angle >= BottomLeft AndAlso Angle < BottomRight Then
                Return Direction.Down
            ElseIf Angle >= BottomRight Then
                Return Direction.Right
            Else
                Throw New NopeException("Irgendwo hat sich ein Wert durchgeschlichen.")
            End If
        End Function

        Private Function DirectionFromDPad(State As XInput.IGamepadState) As Direction?
            Dim PressedStates = New Dictionary(Of Direction, Boolean) From _
                                {
                                    {Direction.Up, State.GetButtonIsDown(XInput.Button.DPadUp)}, _
                                    {Direction.Down, State.GetButtonIsDown(XInput.Button.DPadDown)}, _
                                    {Direction.Left, State.GetButtonIsDown(XInput.Button.DPadLeft)}, _
                                    {Direction.Right, State.GetButtonIsDown(XInput.Button.DPadRight)}
                                }
            Select Case PressedStates.AsEnumerable.Count(Function(i) i.Value)
                Case 0
                    Return Nothing
                Case 1
                    Return PressedStates.Single(Function(i) i.Value).Key
                Case Else
                    Return Nothing
            End Select
        End Function

    End Class

End Namespace
