﻿
#Const EnableXna = False

Namespace XInput.Xna

    Public Structure XnaGamepadStateAdapter
        Implements IGamepadState

        Public ReadOnly Property IsConnected As Boolean Implements IGamepadState.IsConnected
            Get
#If EnableXna Then
                Return State.IsConnected
#Else
                Return Nothing
#End If
            End Get
        End Property

        Public ReadOnly Property PacketNumber As Integer Implements IGamepadState.PacketNumber
            Get
#If EnableXna Then
                Return State.PacketNumber
#Else
                Return Nothing
#End If
            End Get
        End Property

        Public Function GetButtonIsDown(Button As Button) As Boolean Implements IGamepadState.GetButtonIsDown
#If EnableXna Then
            Return State.IsButtonDown(XnaButtons(Button))
#Else
            Return Nothing
#End If
        End Function

        Public Function GetStickValue(Stick As Stick) As Vector2D Implements IGamepadState.GetStickValue
#If EnableXna Then
            Select Case Stick
                Case XInput.Stick.Left
                    Return New Vector2D(State.ThumbSticks.Left.X, State.ThumbSticks.Left.Y)
                Case XInput.Stick.Right
                    Return New Vector2D(State.ThumbSticks.Right.X, State.ThumbSticks.Right.Y)
                Case Else
                    Throw New NopeException
            End Select
#Else
            Return Nothing
#End If
        End Function

        Public Function GetTriggerValue(Trigger As Trigger) As Double Implements IGamepadState.GetTriggerValue
#If EnableXna Then
            Select Case Trigger
                Case XInput.Trigger.Left
                    Return State.Triggers.Left
                Case XInput.Trigger.Right
                    Return State.Triggers.Right
                Case Else
                    Throw New NopeException
            End Select
#Else
            Return Nothing
#End If
        End Function

        Public Shared Function GetState(PlayerIndex As PlayerIndex) As XnaGamepadStateAdapter
#If EnableXna Then
            Return New XnaGamepadStateAdapter With {._State = Microsoft.Xna.Framework.Input.GamePad.GetState(XnaPlayerIndices(PlayerIndex))}
#Else
            Return Nothing
#End If
        End Function

        Public Shared Sub DoLoadTest()
#If EnableXna Then
            Try
                XnaLoadTest()
            Catch ex As Exception
                MessageBox.Show(String.Format("XNA Bibliothek läuft nicht. {1}: {2}{0}{3}", Environment.NewLine, ex.GetType.Name, ex.Message, ex.StackTrace))
                Throw
            End Try
#End If
        End Sub

#If EnableXna Then

        Private Shared Sub XnaLoadTest()
            Microsoft.Xna.Framework.Input.GamePad.GetState(Microsoft.Xna.Framework.PlayerIndex.One)
        End Sub

        Private _State As Microsoft.Xna.Framework.Input.GamePadState
        Public ReadOnly Property State As Microsoft.Xna.Framework.Input.GamePadState
            <DebuggerStepThrough()>
            Get
                Return _State
            End Get
        End Property

        Private Shared ReadOnly XnaButtons As New Dictionary(Of Button, Microsoft.Xna.Framework.Input.Buttons) From _
        {
            {Button.A, Microsoft.Xna.Framework.Input.Buttons.A},
            {Button.B, Microsoft.Xna.Framework.Input.Buttons.B},
            {Button.X, Microsoft.Xna.Framework.Input.Buttons.X},
            {Button.Y, Microsoft.Xna.Framework.Input.Buttons.Y},
            {Button.LeftBumper, Microsoft.Xna.Framework.Input.Buttons.LeftShoulder},
            {Button.RightBumper, Microsoft.Xna.Framework.Input.Buttons.RightShoulder},
            {Button.DPadUp, Microsoft.Xna.Framework.Input.Buttons.DPadUp},
            {Button.DPadDown, Microsoft.Xna.Framework.Input.Buttons.DPadDown},
            {Button.DPadLeft, Microsoft.Xna.Framework.Input.Buttons.DPadLeft},
            {Button.DPadRight, Microsoft.Xna.Framework.Input.Buttons.DPadRight},
            {Button.Back, Microsoft.Xna.Framework.Input.Buttons.Back},
            {Button.Start, Microsoft.Xna.Framework.Input.Buttons.Start},
            {Button.LeftStickButton, Microsoft.Xna.Framework.Input.Buttons.LeftStick},
            {Button.RightStickButton, Microsoft.Xna.Framework.Input.Buttons.RightStick}
        }

        Private Shared ReadOnly XnaPlayerIndices As New Dictionary(Of PlayerIndex, Microsoft.Xna.Framework.PlayerIndex) From _
        {
            {PlayerIndex.One, Microsoft.Xna.Framework.PlayerIndex.One},
            {PlayerIndex.Two, Microsoft.Xna.Framework.PlayerIndex.Two},
            {PlayerIndex.Three, Microsoft.Xna.Framework.PlayerIndex.Three},
            {PlayerIndex.Four, Microsoft.Xna.Framework.PlayerIndex.Four}
        }

#End If

    End Structure

End Namespace
